unit frmTest_Series;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, IB_Controls, IB_Components, IB_StoredProc, Grids, IB_Grid,
  Mask, IB_UpdateBar, Buttons, ExtCtrls;

type
  TfmTest_Series = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Splitter1: TSplitter;
    Panel4: TPanel;
    btExit: TSpeedButton;
    IB_UpdateBar1: TIB_UpdateBar;
    Panel5: TPanel;
    IB_Grid1: TIB_Grid;
    Splitter2: TSplitter;
    Panel6: TPanel;
    IB_Grid2: TIB_Grid;
    Label4: TLabel;
    Label5: TLabel;
    cn: TIB_Connection;
    IB_Transaction1: TIB_Transaction;
    qryInv_Header: TIB_Query;
    qryInv_Log: TIB_Query;
    sp_CancelInvoice: TIB_StoredProc;
    sp_FreeInvNumber: TIB_StoredProc;
    IB_Transaction2: TIB_Transaction;
    dsInv_Header: TIB_DataSource;
    dsInv_Log: TIB_DataSource;
    SpeedButton1: TSpeedButton;
    Panel7: TPanel;
    Panel8: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    IB_Edit1: TIB_Edit;
    Label3: TLabel;
    Label6: TLabel;
    RadioGroup1: TRadioGroup;
    Panel9: TPanel;
    IB_Grid3: TIB_Grid;
    qry_Dead_Invoice: TIB_Query;
    dsDead_Invoice: TIB_DataSource;
    Label7: TLabel;
    btRecycle: TButton;
    procedure btExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure IB_Grid1GetCellFont(Sender: TObject; ACol, ARow: Integer;
      AState: TGridDrawState; AColor: TColor; const AFont: TFont);
    procedure qryInv_HeaderAfterPost(IB_Dataset: TIB_Dataset);
    procedure btRecycleClick(Sender: TObject);
  private
    { Private declarations }
  public
    procedure CancelInvoice(aInvoice: integer; aReason: string);
    procedure RefreshDatasets;
  end;

var
  fmTest_Series: TfmTest_Series;

implementation

{$R *.DFM}

procedure TfmTest_Series.btExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmTest_Series.FormCreate(Sender: TObject);
begin
  if not cn.Connected then
    cn.Connect;
  RefreshDatasets;
end;

// This click sets off the cancellation of an invoice.  It's not valid to
// cancel an invoice unless there is a reason given for it.  This reason
// gets store into the Invoice Log table.
//
// The serial number of cancelled invoices can't be recycled.
procedure TfmTest_Series.SpeedButton1Click(Sender: TObject);
var
  rsn: string;
begin
  if RadioGroup1.ItemIndex >= 0 then
  begin
    rsn := RadioGroup1.Items[RadioGroup1.ItemIndex];
    CancelInvoice(qryInv_header.FieldByName('INV_NUMBER').AsInteger,rsn);
  end
  else
    ShowMessage('A reason code must be selected.');
end;

procedure TfmTest_Series.CancelInvoice(aInvoice: integer; aReason: string);
begin
  cn.BeginBusy(False);
  with IB_Transaction2 do
  begin
    if InTransaction then
      RollBack;
    StartTransaction;
  end;
  with sp_CancelInvoice do
  begin
    if Prepared then Unprepare;
    Prepare;
    ParamByName('INV_NO').AsInteger := AInvoice;
    ParamByName('RSN').AsString := AReason;
    Execute;
  end;
  with IB_Transaction2 do
  if InTransaction then
  try
    Commit;
  except
    RollBack;
  end;
  RefreshDatasets;
  cn.EndBusy;
end;

procedure TfmTest_Series.RefreshDatasets;
var
  i: integer;
begin
  with IB_Transaction1 do
  begin
    for i := 0 to DatasetCount - 1 do
    begin
      if not Datasets[i].Prepared then
      begin
        Datasets[i].Prepare;
        Datasets[i].Open;
      end
      else
        Datasets[i].Refresh;
    end;
  end;
end;

// In the INV_HEADER dataset, status is only CANCELLED or blank, so this
// colours the cancelled invoices
procedure TfmTest_Series.IB_Grid1GetCellFont(Sender: TObject; ACol,
  ARow: Integer; AState: TGridDrawState; AColor: TColor;
  const AFont: TFont);
begin
  if ( ACol = 5 ) and ( ARow >= 1 ) then
    AFont.Color := clBlue;
end;


procedure TfmTest_Series.qryInv_HeaderAfterPost(IB_Dataset: TIB_Dataset);
begin
  RefreshDatasets;
end;

// This calls the stored procedure that frees up invoice numbers that were
// generated and then lost when an invoice header didn't commit
procedure TfmTest_Series.btRecycleClick(Sender: TObject);
begin
  try
    cn.BeginBusy(False);
    with IB_Transaction2 do
    begin
      if InTransaction then
        RollBack;
      StartTransaction;
    end;
    with sp_FreeInvNumber do
    begin
      if Prepared then Unprepare;
      Prepare;
      ParamByName('INV_NO').AsInteger := qry_Dead_Invoice.FieldByName('INV_NUMBER').AsInteger;
      Execute;
    end;
    with IB_Transaction2 do
    if InTransaction then
    try
      Commit;
    except
      RollBack;
    end;
    RefreshDatasets;
  finally
    cn.EndBusy;
  end;
end;
//------------------------------------------------------------------------------
end.


