/* This script creates the test database for the Test_Series demo       */
/* Copyright Computer Programming Solutions, Mesa, AZ, November 2000    */
/************************************************************************/
/* NOTICE :: If you plan create this database as a Version 6, Dialect 3 */
/* then THE GENERATORS ARE INTEGER-64.                                  */
/* Please replace all INTEGER definitions with NUMERIC(18,0)            */
/************************************************************************/
/* Modify and uncomment the following line: */

CREATE DATABASE 'E:\IBO\TECHTOPICS\TEST_SERIES_APP\TEST.GDB' USER 'SYSDBA' PASSWORD 'masterkey' PAGE_SIZE=4096;

CREATE DOMAIN D_IDENTIFIER INTEGER;

CREATE TABLE INV_HEADER(
  INV_ID D_IDENTIFIER NOT NULL,
  CUST_ID D_IDENTIFIER NOT NULL,
  INV_DATE DATE NOT NULL,
  INV_NUMBER D_IDENTIFIER,
  INV_OPERATOR VARCHAR(128),
  STATUS VARCHAR(20),
  CONSTRAINT PK_INV_HEADER PRIMARY KEY(INV_ID));

CREATE TABLE INV_LOG(
  LOG_ID D_IDENTIFIER NOT NULL,
  INV_NUMBER D_IDENTIFIER,
  INV_ID D_IDENTIFIER,
  STATUS VARCHAR(20),
  REASON VARCHAR(50),
  OPERATOR VARCHAR(128),
  GEN_DATE DATE,
  LOGDATE DATE,
  CONSTRAINT PK_INV_LOG PRIMARY KEY(LOG_ID));

CREATE GENERATOR GEN_INV_HEADER_ID;
CREATE GENERATOR GEN_INV_LOG_ID;
CREATE GENERATOR GEN_INV_NUMBER;

CREATE UNIQUE INDEX UN_INV_HDR_INV_NUMBER
ON INV_HEADER(INV_NUMBER);
CREATE UNIQUE INDEX UN_INV_LOG_INV_NUMBER
ON INV_LOG(INV_NUMBER);

COMMIT;

SET TERM ^;
CREATE TRIGGER BI_INSERT_INV_NUM
FOR INV_HEADER
ACTIVE BEFORE INSERT POSITION 9 AS
DECLARE VARIABLE INV_NO INTEGER;  /* v.6/Dialect 3 make this NUMERIC(18,0) */
BEGIN
  INV_NO = 0;
  SELECT MIN(INV_NUMBER) FROM INV_LOG
  WHERE STATUS='AVAILABLE'
  INTO :INV_NO;
  IF (INV_NO > 0) THEN
    UPDATE INV_LOG SET STATUS=STATUS /* LOCKS THE RECORD */
    WHERE  INV_NUMBER= :INV_NO;
  ELSE
  BEGIN
    INV_NO = GEN_ID(GEN_INV_NUMBER, 1);
    INSERT INTO INV_LOG(INV_NUMBER, GEN_DATE)
    VALUES(:INV_NO, 'NOW');
  END
  NEW.INV_NUMBER = :INV_NO;
END ^

CREATE TRIGGER BI_INSERT_INV_HDR
FOR INV_HEADER
ACTIVE BEFORE INSERT POSITION 8 AS
BEGIN
  IF (NEW.INV_ID IS NULL) THEN
    NEW.INV_ID = GEN_ID(GEN_INV_HEADER_ID, 1);
  IF (NEW.INV_DATE IS NULL) THEN
    NEW.INV_DATE='NOW';
  NEW.INV_OPERATOR = USER;
END ^

CREATE TRIGGER AI_INSERT_INV_NUM
FOR INV_HEADER
ACTIVE AFTER INSERT POSITION 10 AS
BEGIN
  UPDATE INV_LOG SET STATUS='USED', 
  INV_ID = NEW.INV_ID,
  REASON ='',
  OPERATOR = USER,
  LOGDATE = 'NOW'
  WHERE INV_NUMBER = NEW.INV_NUMBER;
END ^

CREATE TRIGGER BI_INSERT_INV_LOG
FOR INV_LOG
ACTIVE BEFORE INSERT POSITION 9 AS
BEGIN
  IF (NEW.LOG_ID IS NULL) THEN
    NEW.LOG_ID = GEN_ID(GEN_INV_LOG_ID, 1);
  NEW.OPERATOR = USER;
END ^

CREATE TRIGGER AD_DELETE_INV_HDR
FOR INV_HEADER
ACTIVE AFTER DELETE POSITION 9 AS
BEGIN
  UPDATE INV_LOG 
    SET INV_ID = NULL,
        STATUS='AVAILABLE',
        REASON='INVOICE DELETED',
        OPERATOR = USER,
        LOGDATE='NOW'
    WHERE INV_ID=OLD.INV_ID;
END ^

CREATE TRIGGER BU_UPDATE_INV_LOG
FOR INV_LOG
ACTIVE BEFORE UPDATE POSITION 0 AS
BEGIN
  NEW.OPERATOR = USER;
  NEW.LOGDATE='NOW';
END ^

CREATE PROCEDURE SP_CANCEL_INVOICE (INV_NO INTEGER, RSN VARCHAR(50))
/* v.6/Dialect 3 make INV_NO NUMERIC(18,0) */
AS
BEGIN
  UPDATE INV_LOG
  SET STATUS = 'CANCELLED',
      REASON = :RSN
  WHERE INV_NUMBER = :INV_NO;
  UPDATE INV_HEADER
  SET STATUS = 'CANCELLED'
  WHERE INV_NUMBER = :INV_NO;
END ^

CREATE PROCEDURE SP_FREE_INV_NUMBER(INV_NO INTEGER) /* v.6/Dialect 3 make this NUMERIC(18,0) */
AS
BEGIN
  UPDATE INV_LOG
  SET INV_ID = NULL,  /* it probably is already NULL but we want to make sure */
        STATUS = 'AVAILABLE',
        REASON = 'INVOICE ABORTED',
        OPERATOR = USER,
        LOGDATE = 'NOW'
  WHERE INV_NUMBER = :INV_NO;
END ^
  

SET TERM ;^

COMMIT;

/* insert some dummy records for a demo of recycling */

INSERT INTO INV_HEADER(CUST_ID) VALUES(1234);
INSERT INTO INV_HEADER(CUST_ID) VALUES(1235);
INSERT INTO INV_HEADER(CUST_ID) VALUES(1236);
INSERT INTO INV_HEADER(CUST_ID) VALUES(1237);
INSERT INTO INV_HEADER(CUST_ID) VALUES(1238);
INSERT INTO INV_HEADER(CUST_ID) VALUES(1239);

COMMIT;

/* delete all the dummy header records just created       */
/* in order to have the delete trigger "free" the numbers */

DELETE FROM INV_HEADER;

COMMIT;

/* update the log records to make them "orphans" */

UPDATE INV_LOG SET STATUS='USED'
                                   REASON='INVOICE ABORTED';

COMMIT;
